Attribute VB_Name = "IteratorCommon"
'  FILE:       IteratorCommon.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functionality common across StepMaster and
'              SMRunOnly, pertaining to iterators
'              Specifically, functions to read iterators records
'              in the workspace, load them in an array and so on.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "IteratorCommon."

Public Const gintMinIteratorSequence As Integer = 0

Public Sub RangeComplete(vntIterators As Variant)
    ' This is a debug procedure
    ' Checks if the from, to and step values are present in
    ' the array
    
    Dim bReset As Byte
    Dim bShift As Byte
    Dim lngIndex As Long
    
    ' Set the three lowest order bits to 1
    bReset = 7
    
    BugAssert IsArray(vntIterators) And Not IsEmpty(vntIterators), _
            "Iterators not specified!"
    
    For lngIndex = LBound(vntIterators) To _
            UBound(vntIterators)
        bShift = 1
        bShift = bShift * (2 ^ (vntIterators(lngIndex).IteratorType - 1))
        
        bReset = bReset Xor bShift
    Next lngIndex
    
    ' Assert that all the elements are present
    BugAssert bReset = 0, "Range not completely specified!"

End Sub
Public Sub LoadIteratorsForWsp(cStepsCol As cArrSteps, _
        ByVal lngWorkspaceId As Long, rstStepsInWsp As Recordset)
    ' Initializes the step records in with all the iterator
    ' values for each step
    
    Dim recIterators As Recordset

    On Error GoTo LoadIteratorsForWspErr
    
#If QUERY_ALL Then
    Dim dtStart As Date

    dtStart = Now
    Set recIterators = ReadWspIterators(lngWorkspaceId)
    
    Call LoadIteratorsArray(cStepsCol, recIterators)
    
    recIterators.Close
    
    BugMessage "QueryAll Read + load took: " & CStr(DateDiff("s", dtStart, Now))
    
#Else
    Dim dtStart As Date
    Dim qyIt As DAO.QueryDef
    Dim sSql As String

    dtStart = Now
    If rstStepsInWsp.RecordCount = 0 Then
        Exit Sub
    End If
    
    ' This method has the advantage that if the steps are queried right, everything else follows
    sSql = "Select step_id, version_no, type, iterator_value, " & _
        " sequence_no " & _
        " from iterator_values " & _
        " where step_id = [s_id] " & _
        " and version_no = [ver_no] "

    ' Order the iterators by sequence within a step
    sSql = sSql & " order by sequence_no "
    
    Set qyIt = dbsAttTool.CreateQueryDef(gstrEmptyString, sSql)
    rstStepsInWsp.MoveFirst
    
    While Not rstStepsInWsp.EOF
            
        qyIt.Parameters("s_id").Value = rstStepsInWsp!step_id
        qyIt.Parameters("ver_no").Value = rstStepsInWsp!version_no
        
        Set recIterators = qyIt.OpenRecordset(dbOpenSnapshot)
        
        Call LoadIteratorsArray(cStepsCol, recIterators)
        recIterators.Close
        
        rstStepsInWsp.MoveNext
    Wend
    
    qyIt.Close
    
    BugMessage "Query step at a time Read + load took: " & CStr(DateDiff("s", dtStart, Now))
    
#End If
    
    Exit Sub
    
LoadIteratorsForWspErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "LoadIteratorsForWsp"
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadRsInArrayFailed, _
            gstrSource, _
            LoadResString(errLoadRsInArrayFailed)
    
End Sub
Private Function ReadWspIterators(ByVal lngWorkspaceId As Long) As Recordset

    ' This function will return a recordset that is populated
    ' with the iterators for all the steps in a given workspace
    
    Dim recIterators As Recordset
    Dim qyIt As DAO.QueryDef
    Dim strSql As String

    On Error GoTo ReadWspIteratorsErr
    gstrSource = mstrModuleName & "ReadWspIterators"
    
    strSql = "Select i.step_id, i.version_no, " & _
        " i.type, i.iterator_value, " & _
        " i.sequence_no " & _
        " from iterator_values i, att_steps a " & _
        " where i.step_id = a.step_id " & _
        " and i.version_no = a.version_no " & _
        " and a.workspace_id = [w_id] " & _
        " and a.archived_flag = [archived] "

    ' Find the highest X-component of the version number
    strSql = strSql & " AND cint( mid( a.version_no, 1, instr( a.version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) = " & _
        " ( select max( cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) ) " & _
        " from att_steps AS d " & _
        " WHERE a.step_id = d.step_id ) "
        
    ' Find the highest Y-component of the version number for the highest X-component
    strSql = strSql & " AND cint( mid( a.version_no, instr( a.version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) + 1 ) ) = " & _
        " ( select max( cint( mid( version_no, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) + 1 ) ) ) " & _
        " from att_steps AS b " & _
        " Where a.step_id = b.step_id " & _
        " AND cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) = " & _
        " ( select max( cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) ) " & _
        " from att_steps AS c " & _
        " WHERE a.step_id = c.step_id ) ) "
    

    ' Order the iterators by sequence within a step
    strSql = strSql & " order by i.step_id, i.sequence_no "
    
    Set qyIt = dbsAttTool.CreateQueryDef(gstrEmptyString, strSql)
    qyIt.Parameters("w_id").Value = lngWorkspaceId
    qyIt.Parameters("archived").Value = False
    
    Set recIterators = qyIt.OpenRecordset(dbOpenSnapshot)
    
    qyIt.Close
    Set ReadWspIterators = recIterators

    Exit Function

ReadWspIteratorsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errReadDataFailed, _
            gstrSource, LoadResString(errReadDataFailed)

End Function
Private Sub LoadIteratorsArray(cStepsCol As cArrSteps, _
        recIterators As Recordset)
    ' Initializes the step records with the iterators for
    ' the step

    Dim cNewIt As cIterator
    Dim cStepRec As cStep
    Dim lngStepId As Long
    
    On Error GoTo LoadIteratorsArrayErr
    gstrSource = mstrModuleName & "LoadIteratorsArray"
    
    If recIterators.RecordCount = 0 Then
        Exit Sub
    End If
    
    recIterators.MoveFirst
    While Not recIterators.EOF
        Set cNewIt = New cIterator
        
        lngStepId = CLng(ErrorOnNullField(recIterators, "step_id"))
        If Not cStepRec Is Nothing Then
            If cStepRec.StepId <> lngStepId Then
                Set cStepRec = cStepsCol.QueryStep(lngStepId)
            End If
        Else
            Set cStepRec = cStepsCol.QueryStep(lngStepId)
        End If
        
        ' Initialize iterator values
        cNewIt.IteratorType = CInt(ErrorOnNullField(recIterators, "type"))
        cNewIt.Value = CStr(ErrorOnNullField(recIterators, "iterator_value"))
        cNewIt.SequenceNo = CInt(ErrorOnNullField(recIterators, "sequence_no"))
        
        ' Add this record to the array of iterators
        cStepRec.LoadIterator cNewIt
        
        Set cNewIt = Nothing
        recIterators.MoveNext
    Wend
    
    Exit Sub
    
LoadIteratorsArrayErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "LoadIteratorsArray"
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadRsInArrayFailed, _
            gstrSource, _
            LoadResString(errLoadRsInArrayFailed)
    
End Sub


