Attribute VB_Name = "WindowsApiSM"
'  FILE:       WindowsApiCommon.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    This module contains wrapper functions for the Windows APIs
'              that are used by StepMaster.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "WindowsApiSM."

Private Declare Function SendMessageLong Lib "user32" Alias "SendMessageA" _
        (ByVal hWnd As Long, _
        ByVal Msg As Long, _
        ByVal wParam As Long, _
        ByVal lParam As Long) As Long
Private Declare Function SendMessageAny Lib "user32" Alias "SendMessageA" _
        (ByVal hWnd As Long, _
        ByVal Msg As Long, _
        ByVal wParam As Long, _
        lParam As Any) As Long
Private Declare Function SendMessage Lib "user32" _
        Alias "SendMessageA" _
        (ByVal hWnd As Long, ByVal wMsg As Long, _
        ByVal wParam As Long, lParam As Any) As Long
Private Declare Function FindWindowEx Lib "user32" _
        Alias "FindWindowExA" _
        (ByVal hWndParent As Long, _
        ByVal hWndChildWindow As Long, _
        ByVal lpClassName As String, _
        ByVal lpsWindowName As String) As Long

' Declarations for registry APIs
Private Declare Function RegCloseKey Lib "advapi32.dll" _
        (ByVal hKey As Long) As Long
Private Declare Function RegOpenKeyEx Lib "advapi32.dll" _
        Alias "RegOpenKeyExA" (ByVal hKey As Long, _
        ByVal lpSubKey As String, ByVal ulOptions As Long, _
        ByVal samDesired As Long, phkResult As Long) As Long
Private Declare Function RegQueryValueEx Lib "advapi32.dll" _
        Alias "RegQueryValueExA" (ByVal hKey As Long, _
        ByVal lpValueName As String, _
        ByVal lpReserved As Long, lpType As Long, _
        lpData As Any, lpcbData As Long) As Long

Private Type LVITEM
   mask         As Long
   iItem        As Long
   iSubItem     As Long
   state        As Long
   stateMask    As Long
   pszText      As String
   cchTextMax   As Long
   iImage       As Long
   lParam       As Long
   iIndent      As Long
End Type

Private Const LVM_FIRST As Long = &H1000
Private Const LVM_SETEXTENDEDLISTVIEWSTYLE As Long = (LVM_FIRST + 54)
Private Const LVM_GETEXTENDEDLISTVIEWSTYLE As Long = (LVM_FIRST + 55)

Private Const LVIF_STATE = &H8
Private Const LVS_EX_FULLROWSELECT As Long = &H20
Private Const LVS_EX_GRIDLINES As Long = &H1
Private Const LVS_EX_CHECKBOXES As Long = &H4
Private Const LVM_GETITEMSTATE As Long = (LVM_FIRST + 44)
Private Const LVM_SETITEMSTATE = (LVM_FIRST + 43)
Private Const LVM_GETITEMTEXT As Long = (LVM_FIRST + 45)
Private Const LVIS_STATEIMAGEMASK As Long = &HF000

' Defined by me to check whether the list box is checked
Private Const SM_LVS_CHECKED = &H2000&

Private Const WM_USER = &H400

' Constants used to change the toolbar style
Private Const TB_SETSTYLE = WM_USER + 56
Private Const TB_GETSTYLE = WM_USER + 57

Private Const TBSTYLE_FLAT = &H800

Public Const HKEY_CURRENT_USER = &H80000001
Public Const HKEY_CLASSES_ROOT = &H80000000
Public Const KEY_QUERY_VALUE = &H1
Public Const REG_SZ = 1
Public Const ERROR_SUCCESS = 0&
Public Const hNull = 0
 
' All these constants are used by procedures that set
' the tabstop and the horizontal extent in the list box
Private Const LB_SETTABSTOPS = &H192
Private Const LB_SETHORIZONTALEXTENT = &H194
Private Const LB_GETHORIZONTALEXTENT = &H193
' Amount of white space after the first column that displays
' the file types
Private Const NUMCHARS = 10
Private Const LB_GETCOUNT = &H18B
Private Const LB_ERR = (-1)
Private Const LB_GETTEXT = &H189
Private Const LB_GETTEXTLEN = &H18A
Private Const LB_GETITEMDATA = &H199
Private Const LB_GETITEMHEIGHT = &H1A1
Private Const CB_FINDSTRINGEXACT = &H158

Private Const BIF_RETURNONLYFSDIRS = 1
Private Const BIF_DONTGOBELOWDOMAIN = 2
Private Const BIF_EDITBOX = 16

' Declarations for the GetStartDir function
'Private Type SHITEMID
'    cb As Integer
'    abID As String
'End Type
'
'Private Type ITEMIDLIST
'    mkid As SHITEMID
'End Type
'
Private Type BrowseInfo
    hwndOwner As Long
    pIDLRoot As Long
'    pIDLRoot As ITEMIDLIST
    pszDisplayName As Long
    lpszTitle As Long
    ulFlags As Long
    lpfnCallback As Long
    lParam As Long
    iImage As Long
End Type

Private Declare Function SHBrowseForFolder Lib "shell32" (lpbi As BrowseInfo) As Long
Private Declare Function SHGetPathFromIDList Lib "shell32" (ByVal pidList As Long, ByVal lpbuffer As String) As Long
Private Declare Function lstrcat Lib "kernel32" Alias "lstrcatA" (ByVal lpString1 As String, ByVal lpString2 As String) As Long

' Used by OpenFile
Private Const SW_SHOWNORMAL = 1
Private Const SHELL_EXECUTE_SUCCESS = 32
Private Const SE_ERR_FNF = 2&
Private Const SE_ERR_PNF = 3&
Private Const SE_ERR_ACCESSDENIED = 5&
Private Const SE_ERR_OOM = 8&
Private Const SE_ERR_SHARE = 26&
Private Const SE_ERR_ASSOCINCOMPLETE = 27&
Private Const SE_ERR_DDETIMEOUT = 28&
Private Const SE_ERR_DDEFAIL = 29&
Private Const SE_ERR_DDEBUSY = 30&
Private Const SE_ERR_NOASSOC = 31&
Private Const SE_ERR_DLLNOTFOUND = 32&
Private Const ERROR_BAD_FORMAT = 11&


Private Const MSG_SE_ERR_FNF                As String = "File not found"
Private Const MSG_SE_ERR_PNF                As String = "Path not found"
Private Const MSG_SE_ERR_ACCESSDENIED       As String = "Access denied"
Private Const MSG_SE_ERR_OOM                As String = "Out of memory"
Private Const MSG_SE_ERR_DLLNOTFOUND        As String = "DLL not found"
Private Const MSG_SE_ERR_SHARE              As String = "A sharing violation occurred"
Private Const MSG_SE_ERR_ASSOCINCOMPLETE    As String = "Incomplete or invalid file association"
Private Const MSG_SE_ERR_DDETIMEOUT         As String = "DDE Time out"
Private Const MSG_SE_ERR_DDEFAIL            As String = "DDE transaction failed"
Private Const MSG_SE_ERR_DDEBUSY            As String = "DDE busy"
Private Const MSG_SE_ERR_NOASSOC            As String = "No association for file extension"
Private Const MSG_ERROR_BAD_FORMAT          As String = "Invalid EXE file or error in EXE image"
Private Const MSG_ELSE                      As String = "Unknown error"

Private Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" ( _
        ByVal hWnd As Long, ByVal lpOperation As String, ByVal lpFile As String, _
        ByVal lpParameters As String, ByVal lpDirectory As String, ByVal nShowCmd As Long) As Long


' Used when setting/retrieving the columnwidth of the list view
Private Const LVM_GETCOLUMNWIDTH = LVM_FIRST + 29
Private Const LVM_SETCOLUMNWIDTH = LVM_FIRST + 30

Public Sub SetListViewStyle(lvwList As comctllib.ListView, _
        blnCheck As Boolean)
    ' Sets the style property for the passed in list view control
    ' to include or exclude checkboxes based on the value of the passed
    ' in indicator

    Dim lngStyle As Long
    Dim lngReturn As Long
    
    On Error GoTo SetListViewStyleErr
    
    ' Set/remove the extended bit to include/exclude the checkbox in the list view
    lngStyle = IIf(blnCheck, LVS_EX_CHECKBOXES, 0)
    
    'set the new ListView style
    lngReturn = SendMessageLong(lvwList.hWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, _
            0&, lngStyle)

    Exit Sub
    
SetListViewStyleErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "SetListViewStyle"
    On Error GoTo 0
    Err.Raise vbObjectError + errSetListViewStyleFailed, _
            gstrSource, _
            LoadResString(errSetListViewStyleFailed)
    
End Sub

Public Sub SetChecked(ByVal lvwList As comctllib.ListView, _
        ByVal lstStep As comctllib.ListItem, _
        ByVal blnChecked As Boolean)
    ' Sets/clears the checkbox in the list view
    
    Dim LV As LVITEM

    On Error GoTo SetCheckedErr
    
    With LV
        .mask = LVIF_STATE
        .state = IIf(blnChecked, &H2000, &H1000)
        .stateMask = LVIS_STATEIMAGEMASK
    End With
    
    SendMessageAny lvwList.hWnd, LVM_SETITEMSTATE, lstStep.Index - 1, LV

    Exit Sub
    
SetCheckedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "SetChecked"
    On Error GoTo 0
    Err.Raise errSetCheckedFailed, gstrSource, _
            LoadResString(errSetCheckedFailed)
    
End Sub
Public Function GetChecked(ByVal lvwList As comctllib.ListView, _
        ByVal lstStep As comctllib.ListItem) As Boolean
    ' Returns True if the checkbox in the passed in listitem is
    ' enabled, returns False otherwise
    
    Dim lngReturn As Long

    On Error GoTo GetCheckedErr
    
    lngReturn = SendMessageAny(lvwList.hWnd, LVM_GETITEMSTATE, _
            lstStep.Index - 1, LVIS_STATEIMAGEMASK)

    ' When an item is checked, the LVM_GETITEMSTATE call
    ' returns 8192 (&H2000&).
    GetChecked = IIf(lngReturn And SM_LVS_CHECKED, True, False)
    
    Exit Function
    
GetCheckedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "GetChecked"
    On Error GoTo 0
    Err.Raise errGetCheckedFailed, gstrSource, LoadResString(errGetCheckedFailed)
    
End Function
Public Sub ProcessListViewClick(ByVal lvwSteps As comctllib.ListView)
    ' Called when a click event occurs on the list view control
    ' This procedure is necessary since an ItemClick event does not
    ' occur when the user clicks on the checkbox in the list view
    ' It checks each node in the list view and compares the check box
    ' value against the enabled flag which is stored in the tag
    ' property in each node. If the values are different, calls a
    ' procedure to toggle the enabled property for the node.
    
    Dim intIndex As Long
    Dim mItem As comctllib.ListItem

    On Error GoTo ProcessListViewClickErr
    
    ' Get the current ListView style
    If SendMessageLong(lvwSteps.hWnd, LVM_GETEXTENDEDLISTVIEWSTYLE, 0&, 0&) = 0 Then
        ' There are no checkboxes in the display - exit
        Exit Sub
    End If
    
    For intIndex = 1 To lvwSteps.ListItems.Count
        Set mItem = lvwSteps.ListItems(intIndex)
        If CBool(mItem.Tag) <> GetChecked(lvwSteps, mItem) Then

            ' Since the enabled property has changed, call a procedure
            ' to update it in the array
            Call SetEnabledForStep(mItem)

        End If
    Next intIndex
    
    Exit Sub
    
ProcessListViewClickErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ProcessListViewClick"
    Call ShowError(errUnableToProcessListViewClick)
    
End Sub


'Public Const GW_OWNER = 4
'Public Const GWL_STYLE = -16
'Public Const WS_DISABLED = &H8000000
'Public Const WM_CANCELMODE = &H1F
'Public Const WM_CLOSE = &H10
'
'
Public Sub OpenFile(strFileToOpen As String)
    ' Shells the program associated with the passed in file
    
    Dim lReturn As Long
    Dim Msg As String
    
    On Error GoTo OpenFileErr
    
    If Not StringEmpty(strFileToOpen) Then
        lReturn = ShellExecute(vbNull, vbNullString, strFileToOpen, vbNullString, vbNullString, SW_SHOWNORMAL)
        
        If lReturn <= SHELL_EXECUTE_SUCCESS Then
            'There was an error
            Select Case lReturn
                Case SE_ERR_FNF
                    Msg = MSG_SE_ERR_FNF
                Case SE_ERR_PNF
                    Msg = MSG_SE_ERR_PNF
                Case SE_ERR_ACCESSDENIED
                    Msg = MSG_SE_ERR_ACCESSDENIED
                Case SE_ERR_OOM
                    Msg = MSG_SE_ERR_OOM
                Case SE_ERR_DLLNOTFOUND
                    Msg = MSG_SE_ERR_DLLNOTFOUND
                Case SE_ERR_SHARE
                    Msg = MSG_SE_ERR_SHARE
                Case SE_ERR_ASSOCINCOMPLETE
                    Msg = MSG_SE_ERR_ASSOCINCOMPLETE
                Case SE_ERR_DDETIMEOUT
                    Msg = MSG_SE_ERR_DDETIMEOUT
                Case SE_ERR_DDEFAIL
                    Msg = MSG_SE_ERR_DDEFAIL
                Case SE_ERR_DDEBUSY
                    Msg = MSG_SE_ERR_DDEBUSY
                Case SE_ERR_NOASSOC
                    Msg = MSG_SE_ERR_NOASSOC
                Case ERROR_BAD_FORMAT
                    Msg = MSG_ERROR_BAD_FORMAT
                Case Else
                    Msg = MSG_ELSE
            End Select
            Call ShowMessageStr(Msg)
            ShowError errOpenFileFailed
        End If
    End If
    
    Exit Sub
    
OpenFileErr:
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "OpenFile"
    ShowError errOpenFileFailed

End Sub


Public Function GetStartDir(strDefault As String) As String
    ' Displays the browse start directory dialog box to the user
    
    Dim lpIDList As Long
    Dim sBuffer As String
    Dim szTitle As String
    Dim tStartDir(MAX_PATH) As Byte
    Dim tBrowseInfo As WindowsApiSM.BrowseInfo
'    Dim tDtl As WindowsApiSM.SHITEMID
'    Dim tList As WindowsApiSM.ITEMIDLIST
    
    On Error GoTo GetStartDirErr
    gstrSource = mstrModuleName & "GetStartDir"
            
'    ' Initialize start directory
'    tDtl.abID = strDefault
'    tDtl.cb = LenB(tDtl.cb) + LenB(strDefault)
'    tList.mkid = tDtl
    
    szTitle = "Enter Start Directory"
    With tBrowseInfo
        .lpszTitle = lstrcat(szTitle, "")
        .ulFlags = BIF_RETURNONLYFSDIRS + BIF_DONTGOBELOWDOMAIN + BIF_EDITBOX
'        .pIDLRoot = tList
    End With
    
    lpIDList = SHBrowseForFolder(tBrowseInfo)
    If (lpIDList) Then
        'Set up buffer area for API function call return
        sBuffer = Space(MAX_PATH)
        SHGetPathFromIDList lpIDList, sBuffer
        sBuffer = Left(sBuffer, InStr(sBuffer, vbNullChar) - 1)
    Else
        WriteError errApiFailed, gstrSource
        sBuffer = strDefault
    End If
            
    GetStartDir = sBuffer
    Exit Function
    
GetStartDirErr:
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "GetStartDir"
    WriteError errApiFailed
    On Error GoTo 0
    Err.Raise vbObjectError + errApiFailed, gstrSource, _
            LoadResString(errApiFailed)

End Function
Public Sub SetScrollBarsInList(frmFormToSet As Form, lstListToSet As ListBox)
    ' Displays horizontal scroll bars on the passed in list
    ' box, if the horizontal extent goes beyond the width of
    ' the list
    
    Dim lngRetval As Long
    Dim lngWidth As Long
    Dim lngMaxWidth As Long
    Dim lngCount As Long
    Dim lngIndex As Long
    Dim strList As String
    Dim strMaxList As String

    On Error GoTo SetScrollBarsInListErr
    
    ' Get the count of items in the listbox
    lngCount = SendMessage(lstListToSet.hWnd, LB_GETCOUNT, 0&, 0&)
    If lngCount = LB_ERR Then
        WriteError errUnableToDetListProperty, gstrSource
        Exit Sub
    End If
        
    For lngIndex = 0 To lngCount - 1
        lngRetval = SendMessage(lstListToSet.hWnd, LB_GETTEXTLEN, lngIndex, 0&)
        If lngCount = LB_ERR Then
            WriteError errUnableToGetListData, gstrSource
            Exit Sub
        End If
        
        strList = String$(lngRetval + 1, 0)
        lngCount = SendMessage(lstListToSet.hWnd, LB_GETTEXT, lngIndex, ByVal strList)
        If lngCount = LB_ERR Then
            WriteError errUnableToGetListData, gstrSource
            Exit Sub
        End If
        
        ' Strlist has an extra character at the end due to the
        ' null terminator, but never mind
        If Len(strList) > Len(strMaxList) Then
            strMaxList = strList
        End If
    
    Next lngIndex
        
    ' Add a buffer of twice the column width to the longest
    ' string
    lngMaxWidth = frmFormToSet.TextWidth(strMaxList & _
            String$(2 * NUMCHARS, 0))
    
    ' Set the horizontal extent depending on the longest
    ' entry in the listbox
    ' This will produce a horizontal scroll bar on the list box.
    ' This message requires coordinates in pixels
    lngWidth = CLng(lngMaxWidth / Screen.TwipsPerPixelX)
    lngRetval = SendMessage(lstListToSet.hWnd, LB_SETHORIZONTALEXTENT, lngWidth, 0&)
    
    lstListToSet.Refresh

    Exit Sub
    
SetScrollBarsInListErr:
    Call LogErrors(Errors)
    ' This is not a critical error - allow further processing
    ' to continue

End Sub
Public Function GetListEntryHeight(lstListToSet As ListBox) As Long
    ' Determines the height of an entry in the list box
    ' control
    Dim lngReturn As Long
    
    On Error GoTo GetListEntryHeightErr
    gstrSource = mstrModuleName & "GetListEntryHeight"
    
    GetListEntryHeight = 0
    
    ' Get the count of items in the listbox
    lngReturn = SendMessage(lstListToSet.hWnd, LB_GETITEMHEIGHT, 0&, 0&)
    If lngReturn = LB_ERR Then
        WriteError errUnableToDetListProperty, gstrSource
    Else
        GetListEntryHeight = lngReturn
    End If
        
    Exit Function
    
GetListEntryHeightErr:
    Call LogErrors(Errors)
    ' This is not a critical error - allow further processing
    ' to continue

End Function
Public Sub SetTBar97(TBar As Toolbar)
    ' Sets the toolbar style to the cool VB-IDE style!
    
    Dim lTBarStyle As Long
    Dim lTBarHwnd As Long
    Dim strApiErr As String

    On Error GoTo SetTBar97Err
    gstrSource = mstrModuleName & "SetTBar97"
    
    lTBarHwnd = FindWindowEx(TBar.hWnd, 0&, "ToolbarWindow32", vbNullString)
    If lTBarHwnd = hNull Then
        strApiErr = ApiError(Err.LastDllError)
        WriteError errSetTBar97Failed, gstrSource, strApiErr
    End If
    
    lTBarStyle = SendMessage(lTBarHwnd, TB_GETSTYLE, 0&, 0&)
    lTBarStyle = lTBarStyle Or TBSTYLE_FLAT
    SendMessage lTBarHwnd, TB_SETSTYLE, 0, lTBarStyle
    
    TBar.Refresh
        
    Exit Sub
    
SetTBar97Err:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "SetTBar97"
    ' Not a critical error - log a message to the log and resume
    WriteError errSetTBar97Failed, gstrSource
    
End Sub
Public Function GetRegKey(ByVal strKey As String, _
        ByVal strItem As String, _
        Optional ByVal hRootKey As Long = HKEY_CLASSES_ROOT) _
        As String
    ' Returns the registry entry for the passed in key
    
    Dim hKey As Long
    Dim lngReturn As Long
    Dim lngType As Long
    Dim lngLength As Long
    Dim strValue As String
    
    On Error GoTo GetRegKeyErr
    gstrSource = mstrModuleName & "GetRegKey"
    
    ' Open the passed in subkey
    lngReturn = RegOpenKeyEx(hRootKey, strKey, 0, _
            KEY_QUERY_VALUE, hKey)
    If lngReturn > 0 Then
        WriteError errApiFailed, gstrSource, _
                ApiError(lngReturn)
        On Error GoTo 0
        Err.Raise vbObjectError + errApiFailed, _
            gstrSource, _
            LoadResString(errApiFailed)
    End If
    
    ' Build a buffer that is long enuf to hold the return value
    strValue = String$(255, 0)
    ' and initialize it's length
    lngLength = Len(strValue) + 1
    
    ' Query the registry for the given item under the subkey
    lngReturn = RegQueryValueEx(hKey, strItem, 0&, lngType, _
            ByVal strValue, lngLength)
    If lngReturn = ERROR_SUCCESS Then
        ' Filter out the registry entry - note that the length
        ' populated by the API includes the null terminator
        GetRegKey = Left(strValue, lngLength - 1)
        
    Else
        ' Close the registry key before raising an error
        RegCloseKey hKey
        
        WriteError errApiFailed, gstrSource, _
                ApiError(lngReturn)
        On Error GoTo 0
        Err.Raise vbObjectError + errApiFailed, _
            gstrSource, _
            LoadResString(errApiFailed)
    End If
            
    ' Close the registry key!
    RegCloseKey hKey
    
    Exit Function
    
GetRegKeyErr:
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "GetRegKey"
    WriteError errApiFailed, gstrSource, strKey
    On Error GoTo 0
    Err.Raise vbObjectError + errApiFailed, _
        gstrSource, _
        LoadResString(errApiFailed)

End Function
Public Function GetColWidth(lvwList As comctllib.ListView, _
        ByVal lIndex As Long) As Long
    
    ' Returns the column width for the passed in column index
    GetColWidth = SendMessage(lvwList.hWnd, LVM_GETCOLUMNWIDTH, lIndex, 0)

End Function
Public Function SetColWidth(lvwList As comctllib.ListView, _
        ByVal iIndex As Long, ByVal lWidth As Long)
    
    Dim lRet As Long
    ' Sets the column width for the passed in column index
    lRet = SendMessage(lvwList.hWnd, LVM_SETCOLUMNWIDTH, iIndex, lWidth)
    BugMessage ApiError(lRet)

End Function

Public Function FindComboEntry(SearchCombo As ComboBox, _
        ByVal SearchString As String) As Integer
        
    ' Returns the position at which the passed in string
    ' occurs in the combo box - returns -1 if it is not found
    
    ' Set the default return value to not found
    FindComboEntry = SendMessage(SearchCombo.hWnd, CB_FINDSTRINGEXACT, -1, ByVal SearchString)
    
End Function

