Attribute VB_Name = "WorkspaceCommon"
'  FILE:       WorkspaceCommon.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functionality common across StepMaster and
'              SMRunOnly, pertaining to workspaces
'              Specifically, functions to read workspace records from
'              the database and so on.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "WorkspaceCommon."


Public Function GetWorkspaceDetails( _
    Optional ByVal WorkspaceId As Long, _
    Optional WorkspaceName As String = gstrEmptyString _
    ) As Variant
    ' Depending on the passed in parameter, it returns
    ' either the workspace name or the workspace identifier
    ' in a variant. The calling function must convert the
    ' return value to the appropriate type
   
    Dim rstWorkspace As Recordset
    Dim qyWsp As DAO.QueryDef
    Dim strSql As String
    Dim cTempStr As cStringSM

    On Error GoTo GetWorkspaceDetailsErr
    gstrSource = mstrModuleName & "GetWorkspaceDetails"
    
    If WorkspaceId = 0 And _
            WorkspaceName = gstrEmptyString Then
        On Error GoTo 0
        Err.Raise vbObjectError + errMandatoryParameterMissing, _
                gstrSource, _
                LoadResString(errMandatoryParameterMissing)
    End If
    
    Set cTempStr = New cStringSM
    
    If WorkspaceId = 0 Then
        strSql = " Select workspace_id from att_workspaces " & _
                " where workspace_name = [w_name] "
        Set qyWsp = dbsAttTool.CreateQueryDef(gstrEmptyString, strSql)
        qyWsp.Parameters("w_name").Value = WorkspaceName
    Else
        strSql = " Select workspace_name from att_workspaces " & _
                " where workspace_id = [w_id] "
        Set qyWsp = dbsAttTool.CreateQueryDef(gstrEmptyString, strSql)
        qyWsp.Parameters("w_id").Value = WorkspaceId
    End If
    
    Set cTempStr = Nothing
    
    Set rstWorkspace = qyWsp.OpenRecordset(dbOpenForwardOnly)
    
    If rstWorkspace.RecordCount <> 0 Then
        GetWorkspaceDetails = rstWorkspace.Fields(0)
    Else
        rstWorkspace.Close
        qyWsp.Close
        On Error GoTo 0
        Err.Raise vbObjectError + errInvalidWorkspaceData, _
                gstrSource, _
                LoadResString(errInvalidWorkspaceData)
    End If
    
    rstWorkspace.Close
    qyWsp.Close
    Exit Function
    
GetWorkspaceDetailsErr:
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "GetWorkspaceDetails"
    On Error GoTo 0
    Err.Raise vbObjectError + errGetWorkspaceDetailsFailed, _
            gstrSource, _
            LoadResString(errGetWorkspaceDetailsFailed)

End Function

Public Sub ReadStepsInWorkspace(rstStepsInWorkSpace As Recordset, _
        qySteps As DAO.QueryDef, _
        Optional lngWorkspaceId As Long = glInvalidId, _
        Optional dbLoad As DAO.Database = Nothing, _
        Optional ByVal bSelectArchivedRecords As Boolean = False)

    ' This function will populate the passed in recordset with
    ' all the steps for a given workspace (if one is passed in, else all workspaces)
    
    Dim strSql As String

    On Error GoTo ReadStepsInWorkspaceErr
    
    ' Create a recordset object to retrieve all steps for
    ' the given workspace
    strSql = "Select step_id, step_label, step_file_name, step_text, " & _
        " start_directory, version_no, workspace_id, " & _
        " parent_step_id, parent_version_no, " & _
        " sequence_no, step_level, " & _
        " enabled_flag, degree_parallelism, " & _
        " execution_mechanism, " & _
        " failure_details, continuation_criteria, " & _
        " global_flag, archived_flag, " & _
        " output_file_name, " & _
        " error_file_name, iterator_name " & _
        " from att_steps a " & _
        " where "
        
        ' log_file_name,

    If lngWorkspaceId <> glInvalidId Then
        strSql = strSql & " workspace_id = [w_id] AND "
    End If

    If Not bSelectArchivedRecords Then
        strSql = strSql & " archived_flag = [archived] AND "
    End If

    ' Find the highest X-component of the version number
    strSql = strSql & " cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) = " & _
        " ( select max( cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) ) " & _
        " from att_steps AS d " & _
        " WHERE a.step_id = d.step_id ) "
        
    ' Find the highest Y-component of the version number for the highest X-component
    strSql = strSql & " AND cint( mid( version_no, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) + 1 ) ) = " & _
        " ( select max( cint( mid( version_no, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) + 1 ) ) ) " & _
        " from att_steps AS b " & _
        " Where a.step_id = b.step_id " & _
        " AND cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) = " & _
        " ( select max( cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) ) " & _
        " from att_steps AS c " & _
        " WHERE a.step_id = c.step_id ) ) "
    
    ' Append the order clause as follows
    ' First, separate all global/non-global steps
    ' Order the worker and manager steps by step_level to
    ' ensure that the parent steps are populated before
    ' any sub-steps within it
    ' Further ordering by parent_step_id and sequence_no
    ' ensures that all the children within a parent are
    ' selected in the necessary order
    strSql = strSql & " order by global_flag, step_level, " & _
        " parent_step_id, sequence_no "
    
    If dbLoad Is Nothing Then Set dbLoad = dbsAttTool
    
    ' Create a temporary Querydef object
    Set qySteps = dbLoad.CreateQueryDef(gstrEmptyString, strSql)
    
    ' Initialize the parameter values
    If lngWorkspaceId <> glInvalidId Then
        qySteps.Parameters("w_id").Value = lngWorkspaceId
    End If
    
    If Not bSelectArchivedRecords Then
        qySteps.Parameters("archived").Value = False
    End If
    
    Set rstStepsInWorkSpace = qySteps.OpenRecordset(dbOpenSnapshot)
    
    Exit Sub
    
ReadStepsInWorkspaceErr:
    
    LogErrors Errors
    gstrSource = mstrModuleName & "ReadStepsInWorkspace"
    On Error GoTo 0
    Err.Raise vbObjectError + errReadWorkspaceDataFailed, _
            gstrSource, _
            LoadResString(errReadWorkspaceDataFailed)
    
End Sub
Public Sub ReadWorkspaces(dbLoad As Database, rstWsp As Recordset, _
        qyWsp As DAO.QueryDef, _
        Optional ByVal bSelectArchivedRecords As Boolean = False)

    ' This function will populate the passed in recordset with all workspace records
    
    Dim strSql As String

    On Error GoTo ReadWorkspacesErr
    
    ' Create a recordset object containing all the workspaces
    ' (that haven't been archived) in the database
    strSql = " Select workspace_id, workspace_name, archived_flag " & _
            " from att_workspaces "
            
    If Not bSelectArchivedRecords Then
        strSql = strSql & " where archived_flag = [archived]"
    End If
    strSql = strSql & " order by workspace_name"
    
    Set qyWsp = dbLoad.CreateQueryDef(gstrEmptyString, strSql)
    If Not bSelectArchivedRecords Then
        qyWsp.Parameters("archived").Value = False
    End If
    
    Set rstWsp = qyWsp.OpenRecordset(dbOpenForwardOnly)
    
    Exit Sub
    
ReadWorkspacesErr:
    
    LogErrors Errors
    gstrSource = mstrModuleName & "ReadWorkspaces"
    On Error GoTo 0
    Err.Raise vbObjectError + errReadWorkspaceDataFailed, _
            gstrSource, _
            LoadResString(errReadWorkspaceDataFailed)
    
End Sub
Public Sub ShowWorkspacesInDb(dbLoad As Database)

    Dim recWorkspaces As Recordset
    Dim qryAllWsp As QueryDef
            
    On Error GoTo ShowWorkspacesInDbErr
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    Load frmWorkspaceOpen
    
    Call ReadWorkspaces(dbLoad, recWorkspaces, qryAllWsp)
    
    frmWorkspaceOpen.lstWorkspaces.Clear
        
    ' Load all the workspaces into the listbox
    If recWorkspaces.RecordCount <> 0 Then
        Do
            ' Add the workspace name to the list and store
            ' the corresponding workspace id as the ItemData
            ' property of the item.
            ' The workspace id will be used for all further
            ' processing of the workspace
            frmWorkspaceOpen.lstWorkspaces.AddItem recWorkspaces![workspace_name]
            frmWorkspaceOpen.lstWorkspaces.ItemData(frmWorkspaceOpen.lstWorkspaces.NewIndex) = _
                    recWorkspaces![workspace_id]
            recWorkspaces.MoveNext

        Loop Until recWorkspaces.EOF
    End If
    recWorkspaces.Close
    qryAllWsp.Close
    
    ' Reset the mousepointer
    ShowFree
    
    #If RUN_ONLY Then
        frmWorkspaceOpen.Show vbModal
    #Else
        frmWorkspaceOpen.Show vbModal, frmMain
    #End If
    
    Exit Sub
    
ShowWorkspacesInDbErr:
    LogErrors Errors
    Call ShowFree
    Err.Raise vbObjectError + errProgramError, mstrModuleName & "ShowWorkspacesInDb", _
            LoadResString(errProgramError)

End Sub
Private Sub ReadWorkspaceParameters(lngWorkspaceId As Long, _
    rstWorkSpaceParameters As Recordset, _
    qyWspParams As DAO.QueryDef)

    ' Will populate the recordset with all the parameters for
    ' a given workspace
    
    Dim strSql As String
    
    On Error GoTo ReadWorkspaceParametersErr
    
    strSql = "Select parameter_id, parameter_name, " & _
            " parameter_value, workspace_id, parameter_type, description " & _
            " from workspace_parameters " & _
            " where workspace_id = [w_id] " & _
            " order by parameter_name, parameter_value "

    ' Create a temporary Querydef object and initialize
    ' it's parameter values
    Set qyWspParams = dbsAttTool.CreateQueryDef(gstrEmptyString, strSql)
    qyWspParams.Parameters("w_id").Value = lngWorkspaceId
    
    Set rstWorkSpaceParameters = qyWspParams.OpenRecordset(dbOpenSnapshot)
    
    Exit Sub
    
ReadWorkspaceParametersErr:
    
    LogErrors Errors
    gstrSource = mstrModuleName & "ReadWorkspaceParameters"
    On Error GoTo 0
    Err.Raise vbObjectError + errReadWorkspaceDataFailed, _
            gstrSource, _
            LoadResString(errReadWorkspaceDataFailed)
    
End Sub
Private Sub ReadConnections(lngWorkspaceId As Long, rstConns As Recordset, _
    qyConns As DAO.QueryDef)

    ' Will populate the recordset with all the parameters for
    ' a given workspace
    
    Dim strSql As String
    
    On Error GoTo ReadWorkspaceParametersErr
    
    strSql = "Select connection_id, " & _
            " connection_name, connection_value, workspace_id, description, " & _
            " no_count_display, no_execute, parse_query_only, ANSI_quoted_identifiers, " & _
            " ANSI_nulls, show_query_plan, show_stats_time, show_stats_io, " & _
            " parse_odbc_msg_prefixes, row_count, tsql_batch_separator, query_time_out, " & _
            " server_language, character_translation, regional_settings " & _
            " from workspace_connections " & _
            " where workspace_id = [w_id] " & _
            " order by connection_name, connection_value "

    ' Create a temporary Querydef object and initialize
    ' it's parameter values
    Set qyConns = dbsAttTool.CreateQueryDef(gstrEmptyString, strSql)
    qyConns.Parameters("w_id").Value = lngWorkspaceId
    
    Set rstConns = qyConns.OpenRecordset(dbOpenSnapshot)
    
    Exit Sub
    
ReadWorkspaceParametersErr:
    
    LogErrors Errors
    On Error GoTo 0
    Err.Raise vbObjectError + errReadWorkspaceDataFailed, _
            mstrModuleName & "ReadConnections", LoadResString(errReadWorkspaceDataFailed)
    
End Sub
Private Sub ReadConnectionDtls(lngWorkspaceId As Long, rstConns As Recordset, _
    qyConns As DAO.QueryDef)

    ' Will populate the recordset with all the connection_dtls records for
    ' a given workspace
    
    Dim strSql As String
    
    On Error GoTo ReadWorkspaceParametersErr
    
    strSql = "Select " & FLD_ID_CONN_NAME & ", " & _
            FLD_CONN_DTL_CONNECTION_NAME & ", " & _
            FLD_CONN_DTL_CONNECTION_STRING & ", " & _
            FLD_ID_WORKSPACE & ", " & _
            FLD_CONN_DTL_CONNECTION_TYPE & _
            " from " & TBL_CONNECTION_DTLS & _
            " where " & FLD_ID_WORKSPACE & " = [w_id] " & _
            " order by " & FLD_CONN_DTL_CONNECTION_NAME

    ' Create a temporary Querydef object and initialize
    ' it's parameter values
    Set qyConns = dbsAttTool.CreateQueryDef(gstrEmptyString, strSql)
    qyConns.Parameters("w_id").Value = lngWorkspaceId
    
    Set rstConns = qyConns.OpenRecordset(dbOpenSnapshot)
    
    Exit Sub
    
ReadWorkspaceParametersErr:
    
    LogErrors Errors
    On Error GoTo 0
    Err.Raise vbObjectError + errReadWorkspaceDataFailed, _
            mstrModuleName & "ReadConnectionDtls", LoadResString(errReadWorkspaceDataFailed)
    
End Sub

Public Sub ReadWorkspaceData(lngWorkspaceId As Long, _
        cStepsCol As cArrSteps, _
        cParamsCol As cArrParameters, _
        cConsCol As cArrConstraints, _
        cConns As cConnections, _
        cConnDetails As cConnDtls, _
        rstStepsInWsp As Recordset, _
        qyStepsInWsp As DAO.QueryDef, _
        rstParamsInWsp As Recordset, _
        qyParamsInWsp As DAO.QueryDef, _
        rstConns As Recordset, _
        qyConns As DAO.QueryDef, _
        rstConnDtls As Recordset, _
        qyConnDtls As DAO.QueryDef)
    ' Loads the passed in structures with all the data for
    ' the workspace. It also initializes the recordsets
    ' with the step and parameter records for the workspace.
    
    On Error GoTo ReadWorkspaceDataErr
    
    ShowBusy
    
    Call ReadStepsInWorkspace(rstStepsInWsp, qyStepsInWsp, lngWorkspaceId)
    
    ' Load all the steps in the array
    LoadRecordsetInStepsArray rstStepsInWsp, cStepsCol
        
    ' Initialize the steps with all the iterator
    ' records for each step
    Call LoadIteratorsForWsp(cStepsCol, lngWorkspaceId, rstStepsInWsp)
    
    ReadWorkspaceParameters lngWorkspaceId, rstParamsInWsp, qyParamsInWsp
    
    ' Load all the workspace parameters in the array
    LoadRecordsetInParameterArray rstParamsInWsp, cParamsCol
        
    ' Read and load connection strings
    ReadConnections lngWorkspaceId, rstConns, qyConns
    
    LoadRecordsetInConnectionArray rstConns, cConns
        
    ' Read and load connection information
    ReadConnectionDtls lngWorkspaceId, rstConnDtls, qyConnDtls
    
    LoadRSInConnDtlArray rstConnDtls, cConnDetails
        
    ' Finally, load the step constraints collection class with
    ' all the constraints for the steps in the workspace
    cConsCol.LoadConstraints lngWorkspaceId, rstStepsInWsp
    
    ShowFree
    Exit Sub
    
ReadWorkspaceDataErr:
    ' Log the error code raised by Visual Basic
    ShowFree
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "ReadWorkspaceData"
    Err.Raise vbObjectError + errReadWorkspaceDataFailed, _
            gstrSource, _
            LoadResString(errReadWorkspaceDataFailed)
    
End Sub


