VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cRunStep"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:        cRunStep.cls
'               Microsoft TPC-H Kit Ver. 2.7.0-1005
'               Copyright Microsoft, 2008
'               All Rights Reserved
'
'
'  PURPOSE:     This class executes the step that is assigned to the
'               ExecuteStep property. It executes the pre-execution constraints
'               in sequence and then the step itself. At the end it executes
'               the post-execution constraints. Since these steps should always
'               be executed in sequence, each step is only fired on the
'               completion of the previous step.
'  Contact:     Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this class
Private Const mstrModuleName As String = "cRunStep."
Private mstrSource As String

' Local variable(s) to hold property value(s)
Private mcStep As cStep
Private mcGlobals As cArrSteps
Private mcvntWspPreCons As Variant
Private mcvntWspPostCons As Variant
Private mcvntPreCons As Variant
Private mcvntPostCons As Variant
Private mcIterators As cRunColIt
Private mlngInstanceId As Long ' Identifier for the current instance
Private mlngIndex As Long ' Index value for the current instance
Private mstrCommand As String ' The command string
Private msRunStepDtl As String ' Step text/file name that will go into the run_step_details table
Private mblnAbort As Boolean   ' Set to True when the user aborts the run
Private msOutputFile As String
Private msErrorFile As String
Private miStatus As InstanceStatus
Private mcVBErr As cVBErrorsSM
Public WspParameters As cArrParameters
Public WspConnections As cConnections
Public WspConnDtls As cConnDtls

Private WithEvents mcTermProcess As cTermProcess
Attribute mcTermProcess.VB_VarHelpID = -1
Public RunId As Long
Public CreateInputFiles As Boolean
Private msOutputDir As String

' Object that will execute the step
Private WithEvents mcExecObj As EXECUTEDLLLib.Execute
Attribute mcExecObj.VB_VarHelpID = -1

' Holds the step that is currently being executed (constraint or
' worker step)
Private mcExecStep As cStep

Private Const msCompareExe As String = "\diff.exe"

Private Enum NextNodeType
    mintWspPreConstraint = 1
    mintPreConstraint
    mintStep
    mintWspPostConstraint
    mintPostConstraint
End Enum

' Public events to notify the calling function of the
' start and end time for each step
Public Event StepStart(cStepRecord As cStep, _
        dtmStartTime As Currency, InstanceId As Long)
Public Event StepComplete(cStepRecord As cStep, _
        dtmEndTime As Currency, InstanceId As Long, Status As InstanceStatus)
Public Event ProcessStart(cStepRecord As cStep, _
        strCommand As String, dtmStartTime As Currency, _
        InstanceId As Long)
Public Event ProcessComplete(cStepRecord As cStep, _
        dtmStartTime As Currency, InstanceId As Long, lElapsed As Long)

Private Function AppendDiffErrors(sDiffFile As String)
    ' The file containing the errors generated by the diff utility is passed in
    ' These errors are appended to the error file for the step
    
    Dim sTemp As String
    Dim InputFile As Integer
    
    If Not StringEmpty(sDiffFile) Then
        
        InputFile = FreeFile
        Open sDiffFile For Input Access Read As InputFile
        
        Do While Not EOF(InputFile)         ' Loop until end of file.
            Line Input #InputFile, sTemp    ' Read line into variable.
            mcVBErr.LogMessage sTemp
        Loop
    
        Close InputFile
    End If
    
End Function

Private Sub CreateStepTextFile()
    ' Creates a file containing the step text being executed
    On Error GoTo CreateStepTextFileErr
    
    Dim sInputFile As String
    
    If mcExecStep.ExecutionMechanism = gintExecuteShell Then
        sInputFile = GetOutputFile(gsCmdFileSuffix)
    Else
        sInputFile = GetOutputFile(gsSqlFileSuffix)
    End If

    ' Generate a file containing the step text being executed
    If Not StringEmpty(mcExecStep.StepTextFile) Or mcExecStep.ExecutionMechanism = gintExecuteShell Then
        FileCopy mstrCommand, sInputFile
    Else
        Call WriteCommandToFile(mstrCommand, sInputFile)
    End If
    
    Exit Sub
        
CreateStepTextFileErr:
    
    mcVBErr.LogVBErrors

End Sub
Private Function GetOutputFile(strFileExt As String) As String
    ' This function generates the output file name for the step currently being executed
    ' The value of the built-in parameter 'DefaultDir' is appended with the run identifier
    ' for the file location
    ' The step label is used for the file name and a combination of all iterator values
    ' for the step is used to make the output files unique for each instance
    Dim sFile As String
    Dim sIt As String
    Dim lIt As Long
    
    On Error GoTo GetOutputFileErr
    
    sFile = SubstituteParametersIfPossible(mcExecStep.StepLabel)

    sFile = TranslateStepLabel(sFile)
    
    If mcExecStep Is mcStep Then
        ' Use iterators that have been defined for the worker or any of it's managers
        ' to make the error/log file unique for this instance
        For lIt = mcIterators.Count - 1 To 0 Step -1
            sIt = sIt & gsExtSeparator & mcIterators(lIt).Value
        Next lIt
    End If
    sIt = sIt & strFileExt
    
    ' Ensure that the length of the complete path does not exceed 255 characters
    If Len(msOutputDir) + Len(sFile) + Len(sIt) > MAX_PATH Then
        sFile = Mid(sFile, 1, MAX_PATH - Len(sIt) - Len(msOutputDir))
    End If
    GetOutputFile = msOutputDir & sFile & sIt
    Exit Function
    
GetOutputFileErr:

    ' Does not make sense to log error to the error file yet. Write to the project
    ' log and return the step label as default
    GetOutputFile = mcExecStep.StepLabel & gsExtSeparator & strFileExt
    
End Function


Private Sub HandleExecutionError()

    On Error GoTo HandleExecutionError

    ' Log the error code raised by Visual Basic
    miStatus = gintFailed
    mcVBErr.LogVBErrors
    Call mcVBErr.WriteError(errExecuteStepFailed, _
            OptArgs:="Continuation criteria for the step is: " & gsContCriteria(mcStep.ContinuationCriteria))
    
HandleExecutionError:
    
    ' Logging failed - return
    
End Sub

Public Property Get Index() As Long

    Index = mlngIndex
    
End Property
Public Property Let Index(ByVal vdata As Long)
    
    mlngIndex = vdata
    
End Property
Private Function InitializeExecStatus() As InstanceStatus
    Dim sCompareFile As String
    
    On Error GoTo InitializeExecStatusErr
            
    InitializeExecStatus = mcExecObj.StepStatus
    
    If InitializeExecStatus = gintComplete Then
        If Not StringEmpty(mcExecStep.FailureDetails) Then
            ' Compare output to determine whether the step failed
            sCompareFile = GetShortName(SubstituteParameters( _
                    mcExecStep.FailureDetails, mcExecStep.WorkspaceId, mcIterators, _
                    WspParameters))
            InitializeExecStatus = IIf(CompareOutput(sCompareFile, msOutputFile), gintComplete, gintFailed)
        End If
    End If
    
    Exit Function

InitializeExecStatusErr:
    mcVBErr.LogVBErrors
'    Call LogErrors(Errors)
    InitializeExecStatus = mcExecObj.StepStatus
    
End Function
Private Function CompareOutput(sCompareFile As String, sOutputFile As String) As Boolean
    
    Dim sCmpOutput As String
    Dim sDiffOutput As String
    
    On Error GoTo CompareOutputErr
    
    ' Create temporary files to store the file compare output and
    ' the errors generated by the compare function
    sCmpOutput = CreateTempFile()
    sDiffOutput = CreateTempFile()
    
    ' Run the compare utility and redirect it's output and errors
    SyncShell ("cmd /c " & _
            GetShortName(App.Path & msCompareExe) & gstrBlank & _
            sCompareFile & gstrBlank & sOutputFile & _
            " > " & sCmpOutput & " 2> " & sDiffOutput)

    If FileLen(sDiffOutput) > 0 Then
        ' The compare generated errors - append error msgs to the error file
        Call AppendDiffErrors(sDiffOutput)
        CompareOutput = False
    Else
        CompareOutput = (FileLen(sCmpOutput) = 0)
    End If
    
    If Not CompareOutput Then
        mcVBErr.WriteError errDiffFailed
    End If
    
    ' Delete the temporary files used to store the output of the compare and
    ' the errors generated by the compare
    Kill sDiffOutput
    Kill sCmpOutput
    
    Exit Function

CompareOutputErr:
    mcVBErr.LogVBErrors
    CompareOutput = False
    
End Function
Public Property Get InstanceId() As Long

    InstanceId = mlngInstanceId
    
End Property
Public Property Let InstanceId(ByVal vdata As Long)
    
    mlngInstanceId = vdata
    
End Property

Private Function ExecuteConstraint(vntConstraints As Variant, _
        ByRef intLoopIndex As Integer) As Boolean
    
    ' Returns True if there is a constraint in the passed in
    ' array that remains to be executed
    
    If IsArray(vntConstraints) And Not IsEmpty(vntConstraints) Then
        ExecuteConstraint = (LBound(vntConstraints) <= intLoopIndex) And (intLoopIndex <= UBound(vntConstraints))
    Else
        ExecuteConstraint = False
    End If
    
End Function
Private Function NextStep() As cStep
    
    ' Determines which is the next step to be executed - it could
    ' be either a pre-execution step, the worker step itself
    ' or a post-execution step
    
    Dim cConsRec As cConstraint
    Dim cNextStepRec As cStep
    Dim vntStepConstraints As Variant
    
    ' Static variable to remember exactly where we are in the
    ' processing
    Static intIndex As Integer
    Static intNextStepType As NextNodeType
    
    On Error GoTo NextStepErr
    
    If mblnAbort = True Then
        ' The user has aborted the run - do not run any more
        ' processes for the step
        Set NextStep = Nothing
        Exit Function
    End If
        
    If intNextStepType = 0 Then
        ' First time through this function - set the Index and
        ' node type to initial values
        intNextStepType = mintWspPreConstraint
        intIndex = 0
        RaiseEvent StepStart(mcStep, Determine64BitTime(), mlngInstanceId)
    End If
    
    Do
        Select Case intNextStepType
            Case mintWspPreConstraint
                vntStepConstraints = mcvntWspPreCons
                
            Case mintPreConstraint
                vntStepConstraints = mcvntPreCons
                
            Case mintStep
                ' CONS:
                If mcStep.StepType = gintWorkerStep Then
                    Set cNextStepRec = mcStep
                End If
                
            Case mintWspPostConstraint
                vntStepConstraints = mcvntWspPostCons
                
            Case mintPostConstraint
                vntStepConstraints = mcvntPostCons
                
        End Select
        
        If intNextStepType <> mintStep Then
            ' Check if there is a constraint to be executed
            If ExecuteConstraint(vntStepConstraints, intIndex) Then
                ' Get the corresponding step record to be executed
                ' Query the global step record for the current
                ' constraint
                Set cConsRec = vntStepConstraints(intIndex)
                
                Set cNextStepRec = mcGlobals.QueryStep(cConsRec.GlobalStepId)
                intIndex = intIndex + 1
            Else
                If intNextStepType = mintPostConstraint Then
                    ' No more stuff to be executed for the step
                    ' Raise a Done event
                    Set cNextStepRec = Nothing
                    
                    ' Set the next step type to an invalid value
                    intNextStepType = -1
                Else
                    Call NextType(intNextStepType, intIndex)
                End If
            End If
        Else
            ' Increment the step type so we look at the post-
            ' execution steps the next time through
            Call NextType(intNextStepType, intIndex)
        End If
    
    Loop Until (Not cNextStepRec Is Nothing) Or _
            intNextStepType = -1
    
    Set NextStep = cNextStepRec
    
    Exit Function
    
NextStepErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    mstrSource = mstrModuleName & "NextStep"
    Err.Raise vbObjectError + errNextStepFailed, mstrSource, _
        LoadResString(errNextStepFailed)

End Function
Public Sub Execute()
    ' This procedure is the method that executes the step that
    ' is assigned to the ExecuteStep property. It call a procedure
    ' to determine the next step to be executed.
    ' Then it initializes all the properties of the cExecuteSM object
    ' and calls it's run method to execute it.
    Dim cConn As cConnection
    Dim cRunConnDtl As cConnDtl
    
    On Error GoTo ExecuteErr

    ' If this procedure is called after a step has completed,
    ' we would have to check if we created any temporary files
    ' while executing that step
    If Not mcExecStep Is Nothing Then
        If Not StringEmpty(mcExecStep.StepTextFile) Or mcExecStep.ExecutionMechanism = gintExecuteShell Then
            ' Remove the temporary file that we created while
            ' running this command
            Kill mstrCommand
        End If
        
        Call StepCompleted
    
        ' The VB errors class stores a reference to the Execute class since it uses
        ' a method of the class to write errors to the error log. Hence,
        ' release all references to the Execute object before destroying it.
        Set mcVBErr.ErrorFile = Nothing
        Set mcExecObj = Nothing
        
        ' Delete empty output and error files (generated by shell commands)
        ' (Can be done only after cleaning up cExecObj)
        Call DeleteEmptyOutputFiles
    Else
        ' First time through - initialize the location of output files
        msOutputDir = GetDefaultDir(mcStep.WorkspaceId, WspParameters)
        msOutputDir = msOutputDir & gstrFileSeparator & Trim(Str(RunId)) & gstrFileSeparator
        ' Dummy file since the function expects a file name
        MakePathValid (msOutputDir & "a.txt")
    End If
    
    ' Call a procedure to determine the next step to be executed
    ' - could be a constraint or the step itself
    ' Initialize a module-level variable to the step being
    ' executed
    Set mcExecStep = NextStep
    If mcExecStep Is Nothing Then
        RaiseEvent StepComplete(mcStep, Determine64BitTime(), mlngInstanceId, miStatus)
        ' No more stuff to execute
        Exit Sub
    End If

    Dim sStartDir As String
    
    Set mcExecObj = New EXECUTEDLLLib.Execute
    
    ' The VB errors class uses the WriteError method of the Execute class to write
    ' all VB errors to the error file for the step (this prevents a clash when the
    ' VB errors and Execution errors have to be written to the same log). Hence, store
    ' a reference to the Execute object in mcVBErr
    msErrorFile = GetOutputFile(gsErrorFileSuffix)
    mcExecObj.ErrorFile = msErrorFile
    Call DeleteFile(msErrorFile, bCheckIfEmpty:=False)
    Set mcVBErr.ErrorFile = mcExecObj
    
    If mcExecStep.ExecutionMechanism = gintExecuteShell Then
        sStartDir = Trim$(GetShortName(SubstituteParameters( _
                    mcExecStep.StartDir, mcExecStep.WorkspaceId, mcIterators, WspParameters:=WspParameters)))
        ' Dummy connection object
        Set cConn = New cConnection
        Set cRunConnDtl = New cConnDtl
    Else
        ' Find the connection string value and substitute parameter values in it
        Set cRunConnDtl = WspConnDtls.GetConnectionDtl(mcExecStep.WorkspaceId, mcExecStep.StartDir)
        Set cConn = WspConnections.GetConnection(mcExecStep.WorkspaceId, cRunConnDtl.ConnectionString)
        sStartDir = Trim$(SubstituteParameters(cConn.ConnectionValue, _
                mcExecStep.WorkspaceId, mcIterators, WspParameters:=WspParameters))
    End If
    
    msOutputFile = GetOutputFile(gsOutputFileSuffix)
    Call DeleteFile(msOutputFile, bCheckIfEmpty:=False)
    mcExecObj.OutputFile = msOutputFile
'    mcExecObj.LogFile = GetShortName(SubstituteParameters( _
'            mcExecStep.LogFile, mcExecStep.WorkspaceId, mcIterators, WspParameters:=WspParameters))
    If mcExecStep.ExecutionMechanism = gintExecuteODBC And _
            cRunConnDtl.ConnType = ConnTypeDynamic Then
        Call mcExecObj.DoExecute(BuildCommandString(), sStartDir, mcExecStep.ExecutionMechanism, _
                cConn.NoCountDisplay, cConn.NoExecute, cConn.ParseQueryOnly, cConn.QuotedIdentifiers, _
                cConn.AnsiNulls, cConn.ShowQueryPlan, cConn.ShowStatsTime, cConn.ShowStatsIO, _
                cConn.RowCount, cConn.QueryTimeOut, gstrEmptyString)
    Else
        Call mcExecObj.DoExecute(BuildCommandString(), sStartDir, mcExecStep.ExecutionMechanism, _
                cConn.NoCountDisplay, cConn.NoExecute, cConn.ParseQueryOnly, cConn.QuotedIdentifiers, _
                cConn.AnsiNulls, cConn.ShowQueryPlan, cConn.ShowStatsTime, cConn.ShowStatsIO, _
                cConn.RowCount, cConn.QueryTimeOut, mcExecStep.StartDir)
    End If

    Exit Sub
    
ExecuteErr:
    Call HandleExecutionError
    
    ' We can assume that if we are in this function, a StepStart event has been triggered already.
    RaiseEvent StepComplete(mcStep, Determine64BitTime(), mlngInstanceId, miStatus)
    
End Sub
Private Function BuildCommandString() As String
    ' Process text to be executed - either from the text
    ' field or read it from a file.
    ' This function will always return the command text for ODBC commands
    ' and a file name for Shell commands
    Dim sFile As String
    Dim sCommand As String
    Dim sTemp As String
    
    On Error GoTo BuildCommandStringErr

    If Not StringEmpty(mcExecStep.StepTextFile) Then
        ' Substitute parameter values and environment variables
        ' in the filename
        msRunStepDtl = SubstituteParameters(mcExecStep.StepTextFile, _
                mcExecStep.WorkspaceId, mcIterators, WspParameters:=WspParameters)
                
        sFile = GetShortName(msRunStepDtl)
        
        mstrCommand = SubstituteParametersInText(sFile, mcExecStep.WorkspaceId)
                
        If mcExecStep.ExecutionMechanism = gintExecuteODBC Then
            ' Read the contents of the file and pass it to ODBC
            BuildCommandString = ReadCommandFromFile(mstrCommand)
        Else
            BuildCommandString = mstrCommand
        End If
    Else
        ' Substitute parameter values and environment variables
        ' in the step text
        msRunStepDtl = SubstituteParameters(mcExecStep.StepText, _
                mcExecStep.WorkspaceId, mcIterators, WspParameters:=WspParameters)
        mstrCommand = msRunStepDtl
                
        If mcExecStep.ExecutionMechanism = gintExecuteShell Then
            ' Write the command to a temp file (enables us to execute multiple
            ' commands via the command interpreter)
            mstrCommand = WriteCommandToFile(msRunStepDtl)
            BuildCommandString = mstrCommand
        Else
            BuildCommandString = SQLFixup(msRunStepDtl)
        End If
    End If
    
    If CreateInputFiles Then
        Call CreateStepTextFile
    End If
    
    Exit Function
    
BuildCommandStringErr:
    ' Log the error code raised by the Execute procedure
'    Call LogErrors(Errors)
    mcVBErr.LogVBErrors
    
    On Error GoTo 0
    mstrSource = mstrModuleName & "Execute"
    Err.Raise vbObjectError + errExecuteStepFailed, mstrSource, _
        LoadResString(errExecuteStepFailed) & mstrCommand
    
End Function
Public Sub Abort()

    On Error GoTo AbortErr

    ' Setting the Abort flag to True will ensure that we
    ' don't execute any more processes for this step
    mblnAbort = True
    
    If Not mcExecObj Is Nothing Then
        mcExecObj.Abort
    Else
        ' We are not in the middle of execution yet
    End If

    Exit Sub

AbortErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errProgramError, _
        mstrModuleName & "Abort", _
        LoadResString(errProgramError)

End Sub
Private Sub NextType(ByRef StepType As NextNodeType, _
        ByRef Position As Integer)
        
    StepType = StepType + 1
    Position = 0
    
End Sub
Private Sub StepCompleted()

    On Error GoTo StepCompletedErr

    If Not mcExecStep Is Nothing Then
        If mcExecStep Is mcStep Then
            miStatus = InitializeExecStatus
            If miStatus = gintFailed Then
                ' Create input files if the step failed execution and one hasn't been created already
                If Not CreateInputFiles Then CreateStepTextFile
                Call mcVBErr.WriteError(errExecuteStepFailed, _
                        OptArgs:="Continuation criteria for the step is: " & gsContCriteria(mcStep.ContinuationCriteria))
            End If
        End If
    End If
    
    Exit Sub
    
StepCompletedErr:
    ' Log the error code raised by Visual Basic
    miStatus = gintFailed
    mcVBErr.LogVBErrors
    Call mcVBErr.WriteError(errExecuteStepFailed, _
            OptArgs:="Continuation criteria for the step is: " & gsContCriteria(mcStep.ContinuationCriteria))
    
End Sub
Private Sub DeleteEmptyOutputFiles()

    On Error GoTo DeleteEmptyOutputFilesErr

    ' Delete empty output and error files
    If Not mcExecStep Is Nothing Then
        Call DeleteFile(msErrorFile, bCheckIfEmpty:=True)
        Call DeleteFile(msOutputFile, bCheckIfEmpty:=True)
    End If

    Exit Sub
    
DeleteEmptyOutputFilesErr:
    ' Not a critical error - continue
    
End Sub
Private Function ReadCommandFromFile(strFileName As String) As String

    ' Returns the contents of the passed in file
    
    Dim sCommand As String
    Dim sTemp As String
    Dim InputFile As Integer
    
    On Error GoTo ReadCommandFromFileErr
    
    If Not StringEmpty(strFileName) Then
        
        InputFile = FreeFile
        Open strFileName For Input Access Read As InputFile
        
        Line Input #InputFile, sCommand ' Read line into variable.
        
        Do While Not EOF(InputFile) ' Loop until end of file.
            Line Input #InputFile, sTemp ' Read line into variable.
            sCommand = sCommand & vbCrLf & sTemp
        Loop
    
        Close InputFile
    End If
    
    ReadCommandFromFile = sCommand
    
    Exit Function
    
ReadCommandFromFileErr:
    
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    mstrSource = mstrModuleName & "ReadCommandFromFile"
    On Error GoTo 0
    Err.Raise vbObjectError + errSubValuesFailed, _
            gstrSource, _
            LoadResString(errSubValuesFailed)
    
End Function
Private Function SubstituteParametersIfPossible(strLabel As String)
    
    On Error GoTo SubstituteParametersIfPossibleErr
    
    SubstituteParametersIfPossible = SubstituteParameters(strLabel, _
            mcExecStep.WorkspaceId, mcIterators, WspParameters:=WspParameters)
    Exit Function
            
SubstituteParametersIfPossibleErr:
    SubstituteParametersIfPossible = strLabel

End Function
Private Function SubstituteParametersInText(strFileName As String, _
        lngWorkspace As Long) As String

    ' Reads each line in the passed in file, substitutes parameter
    ' values in the line and writes out the modified line to a
    ' temporary file that we create. The temporary file will be
    ' removed once the step completes execution.
    ' Returns the name of the newly created temporary file.
    
    Dim strTempFile As String
    Dim strTemp As String
    Dim strOutput As String
    Dim InputFile As Integer
    Dim OutputFile As Integer
    
    On Error GoTo SubstituteParametersInTextErr
    
    strTempFile = CreateTempFile()
    
    If Not StringEmpty(strFileName) Then
        
        InputFile = FreeFile
        Open strFileName For Input Access Read As InputFile
        
        OutputFile = FreeFile
        Open strTempFile For Output Access Write As OutputFile
        
        Do While Not EOF(InputFile) ' Loop until end of file.
            Line Input #InputFile, strTemp ' Read line into variable.
            strOutput = SubstituteParameters(strTemp, lngWorkspace, mcIterators, WspParameters:=WspParameters)
            
            If mcExecStep.ExecutionMechanism = gintExecuteODBC Then strOutput = SQLFixup(strOutput)
        
            Print #OutputFile, strOutput
            BugMessage strOutput
        Loop
        
    End If
    
    Close InputFile
    Close OutputFile
    
    SubstituteParametersInText = strTempFile
    
    Exit Function
    
SubstituteParametersInTextErr:
    
    ' Log the error code raised by Visual Basic
'    Call LogErrors(Errors)
    mcVBErr.LogVBErrors
    mstrSource = mstrModuleName & "SubstituteParametersInText"
    On Error GoTo 0
    Err.Raise vbObjectError + errSubValuesFailed, _
            gstrSource, _
            LoadResString(errSubValuesFailed)
    
End Function

Private Function WriteCommandToFile(sCommand As String, Optional sFile As String = gstrEmptyString) As String

    ' Writes the command text to a temporary file
    ' Returns the name of the temporary file
    
    Dim OutputFile As Integer
    
    On Error GoTo WriteCommandToFileErr
    
    If StringEmpty(sFile) Then
        sFile = CreateTempFile()
    End If
    
    OutputFile = FreeFile
    Open sFile For Output Access Write As OutputFile
        
    Print #OutputFile, sCommand
        
    Close OutputFile
    
    WriteCommandToFile = sFile
    
    Exit Function
    
WriteCommandToFileErr:
    
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    mstrSource = mstrModuleName & "WriteCommandToFile"
    On Error GoTo 0
    Err.Raise vbObjectError + errSubValuesFailed, _
            gstrSource, _
            LoadResString(errSubValuesFailed)
    
End Function


Public Property Get WspPreCons() As Variant
    WspPreCons = mcvntWspPreCons
End Property
Public Property Let WspPreCons(ByVal vdata As Variant)
    mcvntWspPreCons = vdata
End Property

Public Property Get WspPostCons() As Variant
    WspPostCons = mcvntWspPostCons
End Property
Public Property Let WspPostCons(ByVal vdata As Variant)
    mcvntWspPostCons = vdata
End Property

Public Property Get PreCons() As Variant
    PreCons = mcvntPreCons
End Property
Public Property Let PreCons(ByVal vdata As Variant)
    mcvntPreCons = vdata
End Property

Public Property Get PostCons() As Variant
    PostCons = mcvntPostCons
End Property
Public Property Let PostCons(ByVal vdata As Variant)
    mcvntPostCons = vdata
End Property

Public Property Set Globals(cRunSteps As cArrSteps)

    Set mcGlobals = cRunSteps
    
End Property
Public Property Set ExecuteStep(cRunStep As cStep)

    Set mcStep = cRunStep
    
End Property
Public Property Get Globals() As cArrSteps

    Set Globals = mcGlobals
    
End Property
Public Property Get ExecuteStep() As cStep

    Set ExecuteStep = mcStep
    
End Property
Public Property Set Iterators(vdata As cRunColIt)

    Set mcIterators = vdata
    
End Property
Private Sub Class_Initialize()
    
    ' Initialize the Abort flag to False
    mblnAbort = False
    Set mcVBErr = New cVBErrorsSM
    Set mcTermProcess = New cTermProcess

End Sub

Private Sub Class_Terminate()

    On Error GoTo Class_TerminateErr
    
    Set mcExecObj = Nothing
    Set mcVBErr = Nothing
    Set mcTermProcess = Nothing
    
    Exit Sub
    
Class_TerminateErr:
    Call LogErrors(Errors)
    
End Sub

Private Sub mcExecObj_Start(ByVal StartTime As Currency)
    ' Raise an event indicating that the step has begun execution
    RaiseEvent ProcessStart(mcExecStep, msRunStepDtl, StartTime, mlngInstanceId)
End Sub
    
Private Sub mcExecObj_Complete(ByVal EndTime As Currency, ByVal Elapsed As Long)

    On Error GoTo mcExecObj_CompleteErr
    
    Debug.Print Elapsed
    RaiseEvent ProcessComplete(mcExecStep, EndTime, mlngInstanceId, Elapsed)
    mcTermProcess.ProcessTerminated
    
    Exit Sub

mcExecObj_CompleteErr:
    Call LogErrors(Errors)
    
End Sub

Private Sub mcTermProcess_TermProcessExists()
    
    On Error GoTo TermProcessExistsErr

    ' Call a procedure to execute the next step, if any
    Call Execute
        
    Exit Sub
    
TermProcessExistsErr:
    ' Log the error code raised by the Execute procedure
    Call LogErrors(Errors)
    
End Sub
